/*
 * e.c
 *
 *  Created on: Apr 12, 2016
 *      Author: lidq
 */

#include <engine.h>
#include <driver.h>

sem_t sem_engine;
pthread_t pthd;
s_engine engine;

//5789.00 189.00 23548.00
double Kp = 3989.00;
double Ki = 216.00;
double Kd = 32875.00;

void engine_start()
{
	for (int i = 0; i < 32; i++)
	{
		signal(i, (void (*)(int)) &engine_exception);
	}

	sem_init(&sem_engine, 0, 0);

	driver_setup();

	engine_reset(&engine);

	pthread_create(&pthd, NULL, (void* (*)(void*)) &engine_balance, NULL);
	pthread_create(&pthd, NULL, (void* (*)(void*)) &engine_test, NULL);

	sem_wait(&sem_engine);
}

void engine_exception()
{
	driver_set_speed(0);

	exit(0);
}

void engine_reset(s_engine *e)
{
	e->dx = 0;
	e->dy = -4.2;
	e->dz = 0;

	e->x = 0;
	e->y = 0;
	e->z = 0;

	e->ax = 0;
	e->ay = 0;
	e->az = 0;

	e->vel = 0;
	e->acc = 0;
}

void engine_test()
{
	s_engine *e = &engine;

	while (true)
	{
		char t;
		double s = 0.0;
		scanf("%c,%lf", &t, &s);
		if (t == '1')
		{
			Kp = s;
		}
		if (t == '2')
		{
			Ki = s;
		}
		if (t == '3')
		{
			Kd = s;
		}
	}
}

void engine_balance()
{
	s_engine *e = &engine;

	usleep(100000);
	mpu6050_setup();
	for (int i = 5; i >= 0; i--)
	{
		usleep(1000000);
		for (int j = 0; j < 100; j++)
		{
			mpu6050_value(&e->x, &e->y, &e->z, &e->ax, &e->ay, &e->az);
		}
		printf("%d\n", i);
	}
	usleep(1000000);
	printf("Go!~\n");

	double et = 0.0, et_1 = 0.0, et_2 = 0.0;

	while (true)
	{
		mpu6050_value(&e->x, &e->y, &e->z, &e->ax, &e->ay, &e->az);

		double angle = e->y + e->dy;
		et_2 = et_1;
		et_1 = et;
		et = angle;

		e->acc = engine_pid(et, et_1, et_2);

		printf("angle: %7.2f\tacc: %7.2f\tvel: %7.2f\tKp: %7.2f\tKi: %7.2f\tKd: %7.2f\n", angle, e->acc,e->vel, Kp, Ki, Kd);

		engine_move(e);

		driver_set_speed(e->vel);

		usleep(ENG_TIMER * 1000);
	}
}

void engine_move(s_engine *e)
{
	double timer = (double) ENG_TIMER / 100.0;
	e->vel = e->vel + e->acc * timer;

	engine_rechk_speed(e);
}

void engine_rechk_speed(s_engine *e)
{
	int MS = MAX_SPEED - 100;
	if (e->vel > MS)
	{
		e->vel = MS;
	}
	if (e->vel < -MS)
	{
		e->vel = -MS;
	}

	if (abs(e->vel) < CRITICAL_SPEED_MIN)
	{
		e->vel = 0;
	}
	else if (e->vel > 0 && e->vel < CRITICAL_SPEED_F)
	{
		e->vel = CRITICAL_SPEED_F;
	}
	else if (e->vel < 0 && e->vel > CRITICAL_SPEED_B)
	{
		e->vel = CRITICAL_SPEED_B;
	}

}

double engine_pid(double et, double et_1, double et_2)
{
	return Kp * (et - et_1) + (Ki * et) + Kd * (et - 2 * et_1 + et_2);
}
